@@ -18,7 +18,7 @@ class ConcatDataset(_ConcatDataset):
     add `get_cat_ids` function.
 
     Args:
-        datasets (list[:obj:`Dataset`]): A list of datasets.
+        datasets (list[:obj:`BaseDataset`]): A list of datasets.
         separate_eval (bool): Whether to evaluate the results
             separately if it is used as validation dataset.
             Defaults to True.
@@ -117,7 +117,7 @@ class RepeatDataset(object):
     epochs.
 
     Args:
-        dataset (:obj:`Dataset`): The dataset to be repeated.
+        dataset (:obj:`BaseDataset`): The dataset to be repeated.
         times (int): Repeat times.
     """
 
@@ -157,9 +157,11 @@ class RepeatDataset(object):
 class ClassBalancedDataset(object):
     r"""A wrapper of repeated dataset with repeat factor.
 
-    Suitable for training on class imbalanced datasets like LVIS. Following
-    the sampling strategy in [#1]_, in each epoch, an image may appear multiple
-    times based on its "repeat factor".
+    Suitable for training on class imbalanced datasets like LVIS. Following the
+    sampling strategy in `this paper`_, in each epoch, an image may appear
+    multiple times based on its "repeat factor".
+
+    .. _this paper: https://arxiv.org/pdf/1908.03195.pdf
 
     The repeat factor for an image is a function of the frequency the rarest
     category labeled in that image. The "frequency of category c" in [0, 1]
@@ -173,27 +175,27 @@ class ClassBalancedDataset(object):
 
     1. For each category c, compute the fraction :math:`f(c)` of images that
        contain it.
-    2. For each category c, compute the category-level repeat factor
+    2. For each category c, compute the category-level repeat factor.
 
         .. math::
             r(c) = \max(1, \sqrt{\frac{t}{f(c)}})
 
+       where :math:`t` is `oversample_thr`.
     3. For each image I and its labels :math:`L(I)`, compute the image-level
-       repeat factor
+       repeat factor.
 
         .. math::
             r(I) = \max_{c \in L(I)} r(c)
 
-    References:
-        .. [#1]  https://arxiv.org/pdf/1908.03195.pdf
+    Each image repeats :math:`\lceil r(I) \rceil` times.
 
     Args:
-        dataset (:obj:`CustomDataset`): The dataset to be repeated.
+        dataset (:obj:`BaseDataset`): The dataset to be repeated.
         oversample_thr (float): frequency threshold below which data is
-            repeated. For categories with `f_c` >= `oversample_thr`, there is
-            no oversampling. For categories with `f_c` < `oversample_thr`, the
-            degree of oversampling following the square-root inverse frequency
-            heuristic above.
+            repeated. For categories with ``f_c`` >= ``oversample_thr``, there
+            is no oversampling. For categories with ``f_c`` <
+            ``oversample_thr``, the degree of oversampling following the
+            square-root inverse frequency heuristic above.
     """
 
     def __init__(self, dataset, oversample_thr):
@@ -215,8 +217,8 @@ class ClassBalancedDataset(object):
         self.flag = np.asarray(flags, dtype=np.uint8)
 
     def _get_repeat_factors(self, dataset, repeat_thr):
-        # 1. For each category c, compute the fraction # of images
-        #   that contain it: f(c)
+        # 1. For each category c, compute the fraction of images
+        #    that contain it: f(c)
         category_freq = defaultdict(int)
         num_images = len(dataset)
         for idx in range(num_images):
@@ -228,15 +230,15 @@ class ClassBalancedDataset(object):
             category_freq[k] = v / num_images
 
         # 2. For each category c, compute the category-level repeat factor:
-        #    r(c) = max(1, sqrt(t/f(c)))
+        #       r(c) = max(1, sqrt(t/f(c)))
         category_repeat = {
             cat_id: max(1.0, math.sqrt(repeat_thr / cat_freq))
             for cat_id, cat_freq in category_freq.items()
         }
 
         # 3. For each image I and its labels L(I), compute the image-level
-        # repeat factor:
-        #    r(I) = max_{c in L(I)} r(c)
+        #    repeat factor:
+        #       r(I) = max_{c in L(I)} r(c)
         repeat_factors = []
         for idx in range(num_images):
             cat_ids = set(self.dataset.get_cat_ids(idx))
@@ -278,7 +280,7 @@ class KFoldDataset:
     and use the fold left to do validation.
 
     Args:
-        dataset (:obj:`CustomDataset`): The dataset to be divided.
+        dataset (:obj:`BaseDataset`): The dataset to be divided.
         fold (int): The fold used to do validation. Defaults to 0.
         num_splits (int): The number of all folds. Defaults to 5.
         test_mode (bool): Use the training dataset or validation dataset.
